import ipkiss3.all as i3
from testbench import Laser, PhotoDiode, create_prbs_function
import numpy as np
from pylab import plt

t0 = 0
dt = 2e-12
t1 = 40e-9
src_bitrate = 0.5e9

f_prbs = create_prbs_function(src_bitrate, t_end=t1, amplitude=0.03535, seed=42)

src = i3.FunctionExcitation(port_domain=i3.ElectricalDomain, excitation_function=f_prbs)

photodiode = PhotoDiode(name="detector")
photodiode.CircuitModel(R=0.2, BW=10e9)

laser = Laser(name="laser")
laser_cm = laser.CircuitModel(
    beta=5e-5,  # Spontaneous emission factor (-)
    tau_e=1.0,  # Electron lifetime (ns)
    gamma=0.35,  # Confinement factor (-)
    v_g=3e8 / 4.0,  # Group speed (m/s)
    sigma_g=2.5 * 1e-16,  # Differential gain (cm^2)
    N_T=1.0e18,  # Carrier density at transparency (cm^-3)
    V=0.1 * 2 * 300,  # Active volume (um^3)
    epsilon_nl=0.0,  # Gain compression factor (-)
    n_sp=2.0,  # Spontaneous emission factor (not used for now)
    alpha_int=25.0,  # Internal losses (including free-carrier absorption, scattering etc) (cm^-1)
    alpha_mirror=1.0 / (2 * 300e-4) * np.log(1 / 1.0 * 1 / 0.05),  # Mirror losses 1/(2L)*ln(1/R1*1/R2) (cm^-1)
    alpha_h=4,
)  # Linewidth enhancement factor (-)

print(f"Laser threshold: {laser_cm.I_th * 1e3} mA")  # For the parameters given above, this should be 17.8450*1e-3

circuit = i3.ConnectComponents(
    child_cells={
        "laser": laser,
        "src": src,
        "photodiode": photodiode,
        "current_out": i3.Probe(port_domain=i3.ElectricalDomain),
    },
    links=[("src:out", "laser:in"), ("laser:out", "photodiode:in"), ("photodiode:anode", "current_out:in")],
)

tb_cm = circuit.CircuitModel()

result = tb_cm.get_time_response(t0=t0, dt=dt, t1=t1, center_wavelength=1.55)

times = result.timesteps

plt.subplot(211)
plt.ylabel("Input Current")
plt.plot(times * 1e9, np.abs(result["current_out"]), "r", linewidth=2, label="laser input")
plt.subplot(212)

plt.ylabel("Output Current")
plt.plot(times * 1e9, np.abs(result["src"]), "g", linewidth=2, label="photodiode output")
plt.xlabel("Time (ns)")
plt.show()
